#ifndef ENGINE_H
#define ENGINE_H

#include "../types.h"

#include <memory>

namespace fd {
namespace engines {

template<typename TransitionModel, typename RewardModel>
class Engine {
public:
    Engine(
        std::shared_ptr<TransitionModel> t_model,
        std::shared_ptr<RewardModel> r_model);
    virtual ~Engine() = default;

    void prepare();
    ValueType get_reward(const State& state);
    Action get_policy_action(const State& state);

    std::shared_ptr<TransitionModel> get_transition_model() const;
    std::shared_ptr<RewardModel> get_reward_model() const;

protected:
    std::shared_ptr<TransitionModel> m_transition_model;
    std::shared_ptr<RewardModel> m_reward_model;
};


template<typename TransitionModel, typename RewardModel>
Engine<TransitionModel, RewardModel>::Engine(
        std::shared_ptr<TransitionModel> model,
        std::shared_ptr<RewardModel> reward)
    : m_transition_model(model)
    , m_reward_model(reward)
{}

template<typename Model, typename RewardModel>
std::shared_ptr<Model>
Engine<Model, RewardModel>::get_transition_model() const
{
    return m_transition_model;
}

template<typename Model, typename RewardModel>
std::shared_ptr<RewardModel>
Engine<Model, RewardModel>::get_reward_model() const
{
    return m_reward_model;
}


}
}

#endif
