#ifndef HEURISTIC_H
#define HEURISTIC_H

#include "evaluation_result.h"
#include "../types.h"

#include <memory>

namespace fd {

namespace storage {
class CompressedArrayHashSet;
}

namespace heuristics {

template<typename Reward>
class Heuristic {
public:
    Heuristic(std::shared_ptr<Reward> reward,
              const storage::CompressedArrayHashSet* state_cache);
    virtual ~Heuristic() = default;

    std::shared_ptr<Reward> get_reward_model() const;
    const storage::CompressedArrayHashSet* get_state_cache() const;

    virtual EvaluationResult evaluate(const State& state) = 0;
protected:
    std::shared_ptr<Reward> m_reward;
    const storage::CompressedArrayHashSet* m_state_cache;
};

template<typename Reward>
Heuristic<Reward>::Heuristic(
        std::shared_ptr<Reward> reward,
        const storage::CompressedArrayHashSet* state_cache)
    : m_reward(reward)
    , m_state_cache(state_cache)
{}

template<typename Reward>
std::shared_ptr<Reward>
Heuristic<Reward>::get_reward_model() const
{
    return m_reward;
}

template<typename Reward>
const storage::CompressedArrayHashSet*
Heuristic<Reward>::get_state_cache() const
{
    return m_state_cache;
}

}
}

#endif
