#include "expcost_reward_model.h"
#include "../../sas_task/probabilistic_sas_task.h"
#include "../../sas_task/fact_pair.h"
#include "../../storage/compressed_array_hash_set.h"
#include "../../storage/array_compressor.h"

#include <vector>

namespace fd {
namespace models {
namespace sas_model {

using storage::compression::CompressedArrayConstReference;

ExpCostRewardModel::ExpCostRewardModel(
    CostType cost_type,
    ValueType inf,
    const std::shared_ptr<sas_task::ProbabilisticSASTask> task,
    const storage::CompressedArrayHashSet& state_cache)
    : m_cost_type(cost_type)
    , m_inf(inf)
    , m_task(task)
    , m_state_cache(state_cache)
{}

ValueType
ExpCostRewardModel::get_immediate_reward(const State&)
{
    return 0;
}

ValueType
ExpCostRewardModel::get_transition_reward(
        const State& ,
        const Action& ,
        const State& )
{
    return 1;
}

bool
ExpCostRewardModel::is_terminal(const State& s)
{
    CompressedArrayConstReference state = m_state_cache.get_compressed_array(s);
    for (int i = m_task->get_num_goals() - 1; i >= 0; i--) {
        const auto& p = m_task->get_goal_fact(i);
        if (state[p.var] != p.value) {
            return false;
        }
    }
    return true;
}

ValueType
ExpCostRewardModel::transform_reward(const ValueType& v)
{
    return -v;
}

ValueType
ExpCostRewardModel::get_min_reward() const
{
    return -m_inf;
}

ValueType
ExpCostRewardModel::get_max_reward() const
{
    return 0;
}

}
}
}

