#include "minprob_reward_model.h"
#include "../../sas_task/probabilistic_sas_task.h"
#include "../../sas_task/fact_pair.h"
#include "../../storage/compressed_array_hash_set.h"
#include "../../storage/array_compressor.h"

#include <vector>

namespace fd {
namespace models {
namespace sas_model {

using storage::compression::CompressedArrayConstReference;

MinProbRewardModel::MinProbRewardModel(
    const std::shared_ptr<sas_task::ProbabilisticSASTask> task,
    const storage::CompressedArrayHashSet& state_cache)
    : m_task(task)
    , m_state_cache(state_cache)
{}

ValueType
MinProbRewardModel::get_immediate_reward(const State& s)
{
    if (is_terminal(s)) {
        return -1;
    }
    return 0;
}

ValueType
MinProbRewardModel::get_transition_reward(
        const State& ,
        const Action& ,
        const State& )
{
    return 0;
}

bool
MinProbRewardModel::is_terminal(const State& s)
{
    CompressedArrayConstReference state = m_state_cache.get_compressed_array(s);
    for (int i = m_task->get_num_goals() - 1; i >= 0; i--) {
        const auto& p = m_task->get_goal_fact(i);
        if (state[p.var] != p.value) {
            return false;
        }
    }
    return true;
}

ValueType
MinProbRewardModel::transform_reward(const ValueType& v)
{
    return -v;
}

ValueType
MinProbRewardModel::get_min_reward() const
{
    return -1;
}

ValueType
MinProbRewardModel::get_max_reward() const
{
    return 0;
}

}
}
}

