#ifndef PROBABILISTIC_SAS_TASK_H
#define PROBABILISTIC_SAS_TASK_H

#include "fact_pair.h"
#include "action.h"
#include "probabilistic_action.h"

#include <vector>
#include <set>
#include <string>
#include <iostream>
#include <memory>

namespace fd {
namespace sas_task {

class SASTask;

class ProbabilisticSASTask
{
public:
    ProbabilisticSASTask(const std::shared_ptr<SASTask> task);
    ProbabilisticSASTask(std::istream& in);
    ~ProbabilisticSASTask() = default;

    unsigned get_num_variables() const;
    std::string get_variable_name(int var) const;
    unsigned get_variable_domain_size(int var) const;
    int get_variable_axiom_layer(int var) const;
    int get_variable_default_axiom_value(int var) const;
    std::string get_fact_name(const FactPair& fact) const;
    bool are_facts_mutex(
        const FactPair& fact1, const FactPair& fact2) const;

    const std::vector<int>& get_initial_state_values() const;
    unsigned get_num_goals() const;
    const FactPair& get_goal_fact(int x) const;

    unsigned get_num_actions() const;
    unsigned get_num_axioms() const;
    const ProbabilisticAction& get_action(int a) const;
    const Action& get_axiom(int a) const;

    bool has_axioms() const;
    bool has_conditional_effects() const;

    const std::set<FactPair>& get_mutexes_for_fact(const FactPair& f) const;

    const std::shared_ptr<SASTask> get_determinization() const;
private:
    void reconstruct_probabilistic_actions(const SASTask& task);

    std::shared_ptr<SASTask> m_sas_task;
    std::vector<ProbabilisticAction> m_prob_actions;
};

}
}

#endif
