#include "compressed_array_hash_set.h"
#include "hash.h"

namespace fd {
namespace storage {

CompressedArrayHashSet::CompressedArrayHashSet(
        const std::vector<int>& bounds)
    : m_compressor(bounds)
    , m_compressed_arrays(m_compressor.get_num_bins())
    , m_id_set(//512244,
            CompressedArrayHasher(m_compressed_arrays, m_compressor.get_num_bins()),
            CompressedArrayEqual(m_compressed_arrays, m_compressor.get_num_bins()))
{}

CompressedArrayHashSet::~CompressedArrayHashSet()
{}

unsigned
CompressedArrayHashSet::size() const
{
    return m_compressed_arrays.size();
}

unsigned
CompressedArrayHashSet::get_num_bytes_per_array() const
{
    return sizeof(compression::Bin) * m_compressor.get_num_bins();
}

compression::CompressedArrayConstReference
CompressedArrayHashSet::get_compressed_array(const Id& id) const
{
    return compression::CompressedArrayConstReference(
            m_compressed_arrays[id],
            m_compressor);
}

const compression::ArrayCompressor&
CompressedArrayHashSet::get_compressor() const
{
    return m_compressor;
}

CompressedArrayHashSet::Id
CompressedArrayHashSet::insert_compressed_get_id(const compression::Bin* data)
{
    m_compressed_arrays.push_back(data);
    return insert_id_or_pop_back();
}

CompressedArrayHashSet::Id
CompressedArrayHashSet::insert_id_or_pop_back()
{
    Id i = m_id_set.size();
    std::pair<int, bool> j = m_id_set.insert(i);
    // auto j = m_id_set.insert(i);
    if (j.second) {
        assert(m_compressed_arrays.size() == (unsigned) m_id_set.size());
        return i;
    }
    m_compressed_arrays.pop_back();
    assert(m_compressed_arrays.size() ==(unsigned)  m_id_set.size());
    // return *(j.first);
    return j.first;
}

CompressedArrayHashSet::CompressedArrayHasher::CompressedArrayHasher(
        const SegmentedArrayVector<compression::Bin>& data,
        unsigned num_bins)
    : data(data), num_bins(num_bins)
{}

int_hash_set::HashType
// std::size_t
CompressedArrayHashSet::CompressedArrayHasher::operator()(const Id& x) const
{
    const compression::Bin *bin = data[x];
    hash::HashState hash_state;
    for (unsigned i = 0; i < num_bins; i++) {
        hash_state.feed(bin[i]);
    }
    return hash_state.get_hash32();
}

CompressedArrayHashSet::CompressedArrayEqual::CompressedArrayEqual(
        const SegmentedArrayVector<compression::Bin>& data,
        unsigned num_bins)
    : data(data), num_bins(num_bins)
{}

bool
CompressedArrayHashSet::CompressedArrayEqual::operator()(const Id& x, const Id& y) const
{
    const compression::Bin *binx = data[x];
    const compression::Bin *biny = data[y];
    for (int i = num_bins - 1; i >= 0; i--) {
        if (binx[i] != biny[i]) {
            return false;
        }
    }
    return true;
}

}
}

