#ifndef STORAGE_STATE_MAP_H
#define STORAGE_STATE_MAP_H

#include "../types.h"
#include <unordered_map>

namespace fd {
namespace storage {

template<typename Value>
class StateMap {
public:
    StateMap();
    StateMap(const StateMap&) = default;
    StateMap(StateMap&&) = default;
    ~StateMap() = default;

    StateMap& operator=(const StateMap&) = default;
    StateMap& operator=(StateMap&&) = default;

    Value& operator[](const State& state);
    void clear();
    bool empty() const;

private:
    std::unordered_map<State, Value> m_map;
};

template<typename Value>
StateMap<Value>::StateMap()
{}

template<typename Value>
Value&
StateMap<Value>::operator[](const State& state)
{
    return m_map[state];
}

template<typename Value>
void
StateMap<Value>::clear()
{
    m_map.clear();
}

template<typename Value>
bool
StateMap<Value>::empty() const
{
    return m_map.empty();
}

}
}

#endif
